/**************************************************************

File Name:	Complex_Vector_Dot_Product.asm
		
Date Modified:	02/17/99	RFG		original version
				05/01/00	RFG		modified format
				06/13/00	RFG		verfied on silicon

Description:
			Subroutine that implements a Complex Vector Dot Product
			given two complex input vectors.

			Equation:	Z = Summation from k=0 to M of X(k)*Y(k)
						  = SUM[(Xr + jXi) * (Yr + jYi)] = SUM[(Xr*Yr - Xi*Yi) + j(Xr*Yi + Xi*Yr)] 

Calling Parameters:
			b0,i0 = address of x input
			b8,i8 = address of y input
			r2 = number of samples - 1
			l0,l8 = 0
			m2 = 2

Assumptions:
			All arrays must start on even address boundaries.
			All arrays must have an even number 32-bit word length (zero  
		    pad if necessary).
			X input is stored in Block 1.
			Instructions and Y input stored in Block 0.

Return Values:
			f12 = Zr (real component of result)
			f13 = Zi (imaginary component of result)

Registers Affected:
			f0,s0,f4,s4,f8,s8,f12,s12,f13,s13
			i0,i8

Cycle Count:
			11 + samples*2 + 3 cache misses
			
Memory Usage:
			Instructions Words (48-bits):
				15 instruction words
			
			Data Words (16, 32, 40, or 64-bits):
				Number of samples locations for the x input buffer (32-bits)
				Number of samples locations for the y input buffer (32-bits)

**************************************************************/

#include    "def21364.h"					/* Symbol Definition File */

.global cx_vec_dot_prod;

/* program memory code */
.section/pm seg_pmco;

cx_vec_dot_prod:
		bit set MODE1 CBUFEN | PEYEN;		/* Circular Buffer Enable, SIMD Mode enable */
		nop;
	
		f8 = 0.0;							/* zero accumulators */
		f12=pass f8;
		f13=pass f12, f0=dm(i0,m2), f4=pm(i8,m9);       	/* Zero accumulators F12/S12, F0 = Xr, S0 = Xi, F4 = Yr, S4 = Yi */
        
lcntr=r2, do macs until lce;				/* FIR loop */
		f8=f0*f4, f13=f8+f13, r0<->s0;						/* F8 = Xr*Yr, S8 = Xi*Yi, F13 = Sum(Xi*Yr), S13 = Sum(Xr*Yi), F0 = Xi, S0 = Xr */
macs:	f8=f0*f4, f12=f8+f12, f0=dm(i0,m2), f4=pm(i8,m9);	/* F8 = Xi*Yr, S8 = Xr*Yi, F12 = Sum(Xr*Yr), S12 = Sum(Xi*Yi) */
															/* F0 = Xr, S0 = Xi, F4 = Yr, S4 = Yi */
		
		f8=f0*f4, f13=f8+f13, r0<->s0;		/* F8 = Xr*Yr, S8 = Xi*Yi, F13 = Sum(Xi*Yr), S13 = Sum(Xr*Yi), F0 = Xi, S0 = Xr */
		f8=f0*f4, f12=f8+f12;				/* F8 = Xi*Yr, S8 = Xr*Yi, F12 = Sum(Xr*Yr), S12 = Sum(Xi*Yi) */
		f13=f8+f13, r0=s12;					/* F13 = Sum(Xi*Yr), S13 = Sum(Xr*Yi), F0 = S12 = Sum(Xi*Yi) */
		f12=f12-f0, r0=s13;					/* F12 = F12-F0 = Sum(Xr*Yr)-Sum(Xi*Yi), F0 = S13 = Sum(Xi*Yr) */

		rts (db), f13=f0+f13;				/* F13 = F0+F13 = Sum(Xr*Yi) + Sum(Xi*Yr) */
		bit clr MODE1 CBUFEN | PEYEN;		/* Circular Buffer disable, SIMD disable */
		nop;

